// 
// www.lebois-racing.fr/en
// For Uno only

// WIRING : IT CAN'T BE CHANGED. SAME AS OPENSFX100
#define StepPin1  8 
#define StepPin2  9
#define StepPin3  10
#define StepPin4  11

#define DirPin1   4
#define DirPin2   5
#define DirPin3   6
#define DirPin4   7

#define RelayPin  A0 //used to control the PSU thougth a relay

#define pulseDelay 2      // mostly set the speed of the actuator : the lower the faster. Don't go below 2 or it will fail.
#define directionDelay 4

byte buffer           = 0 ;    // It takes the value of the serial data
byte buffercount      = 0 ;    // To count where we are in the serial datas
byte commandbuffer[8] = {0};   // To stock the serial datas in the good order.   

unsigned m1Target = 0,   m2Target = 0,   m3Target = 0,   m4Target = 0; 
unsigned m1Position = 0, m2Position = 0, m3Position = 0, m4Position = 0;
byte pulseM1 = B00000000, pulseM2 = B00000000, pulseM3 = B00000000, pulseM4 = B00000000;
int  dir1 = -1, dir2 = -1, dir3 = -1, dir4 = -1;         //Will be used to set the motors direction
byte dirChange = 0;
bool noData = true;
bool servoEnabled = false;

void setup() {
  Serial.begin(921600);    //To communicate with FlyPT. In FlyPT, put that number in "Serial speed".

  pinMode(RelayPin, OUTPUT);
  disableServo();

  pinMode(StepPin1, OUTPUT);
  pinMode(DirPin1, OUTPUT);
  digitalWrite(DirPin1, HIGH);

  pinMode(StepPin2, OUTPUT);
  pinMode(DirPin2, OUTPUT);
  digitalWrite(DirPin2, HIGH);

  pinMode(StepPin3, OUTPUT);
  pinMode(DirPin3, OUTPUT);
  digitalWrite(DirPin3, HIGH); 

  pinMode(StepPin4, OUTPUT);
  pinMode(DirPin4, OUTPUT);
  digitalWrite(DirPin4, HIGH);  


}

void loop() {
  SerialReader();                     //Get the datas from Simtools
  moveMotor();                        //Move the motors
}

void SerialReader() {       // FlyPT output : P<Axis1a><Axis2a><Axis3a><Axis4a>, Data bits : 8 bits, Parity : None, stop bits : 1
  while (Serial.available())
  {
    if (buffercount == 0)
    {
      buffer = Serial.read();

            if (buffer != 'P') {
        if(buffer =='A'){enableServo();}
        if(buffer =='C'){disableServo();}     
                      
        buffercount = 0; // "S" is the marquer. If we read P, the next data is Motor1
      } else {
        buffercount = 1;
      }
    }
    else   //  if(buffercount>=1)
    {
      buffer = Serial.read();
      commandbuffer[buffercount-1] = buffer; // The first value next to "P" is saved in commandbuffer in the place "buffercount"
      buffercount++;
      if (buffercount >= 9) {
          m1Target = commandbuffer[0] * 256 + commandbuffer[1];      //because the communication is 8bits, and the data are 16 bits, their is two bytes to combine
          m2Target = commandbuffer[2] * 256 + commandbuffer[3];
          m3Target = commandbuffer[4] * 256 + commandbuffer[5];
          m4Target = commandbuffer[6] * 256 + commandbuffer[7];
          buffercount = 0;
        break;
      }
    }
  }
}


void moveMotor() {
  directionManager(); //set the directions of each motor
  singleStep(); //make the motor move one step
}

void directionManager() { 

  if ((m1Target > m1Position) && (dir1 == -1)) {
    PORTD &= B11101111;
    dir1 = 1;
    dirChange = 1;
  }

  if ((m1Target < m1Position ) && (dir1 == 1)) {
    PORTD |= B00010000;
    dir1 = -1;
    dirChange = 1;
  }

  if ((m2Target > m2Position) && (dir2 == -1)) {
    PORTD &= B11011111;
    dir2 = 1;
    dirChange = 1;
  }
  
  if ((m2Target < m2Position ) && (dir2 == 1)) {
    PORTD |= B00100000;
    dir2 = -1;
    dirChange = 1;
  }

  if ((m3Target > m3Position) && (dir3 == -1)) {
    PORTD &= B10111111;
    dir3 = 1;
    dirChange = 1;
  }
  
  if ((m3Target < m3Position ) && (dir3 == 1)) {
    PORTD |= B01000000;
    dir3 = -1;
    dirChange = 1;
  }

  if ((m4Target > m4Position) && (dir4 == -1)) {
    PORTD &= B01111111;
    dir4 = 1;
    dirChange = 1;
  }
  
  if ((m4Target < m4Position ) && (dir4 == 1)) {
    PORTD |= B10000000;
    dir4 = -1;
    dirChange = 1;
  }
  
  if (dirChange == 1) {
    delayMicroseconds(directionDelay);
    dirChange = 0;
  }
}

void singleStep() {
  //pulse is 8, 9, 10, 11
  if (m1Target != m1Position) {
    pulseM1 = B00000001;
    m1Position += dir1;
  }
  else{pulseM1 = B00000000;}
  
  if (m2Target != m2Position) {
    pulseM2 = B00000010;
    m2Position += dir2;
  }
  else{pulseM2 = B00000000;}
    
  if (m3Target != m3Position) {
    pulseM3 = B00000100;
    m3Position += dir3;
  }
  else{pulseM3 = B00000000;}

  if (m4Target != m4Position) {
    pulseM4 = B00001000;
    m4Position += dir4;
  }
  else{pulseM4 = B00000000;}
    
  if((pulseM1 == B00000000) && (pulseM2 == B00000000) && (pulseM3 == B00000000) && (pulseM4 == B00000000)){return;}
      PORTB |= pulseM1 | pulseM2 | pulseM3 | pulseM4;
      delayMicroseconds(pulseDelay);
      PORTB &= B11110000;
}

void enableServo(){
    digitalWrite(RelayPin, LOW);
    servoEnabled=true;
}
void disableServo(){
    digitalWrite(RelayPin, HIGH);
}
  
